/***************************************************************************
    copyright            : (C) 2009 by Abdulhaq Lynch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; Version 2 of the License.               *
 *                                                                         *
 ***************************************************************************/

#include "config.h"

#include <QLocale>

ConfigDialog::ConfigDialog() : QWidget()
{
   setupUi(this);

   cmbITLMethod->addItem(QString("Egyptian General Authority of Survey"));
   cmbITLMethod->addItem(QString("University of Islamic Sciences, Karachi (Shaf'i)"));
   cmbITLMethod->addItem(QString("University of Islamic Sciences, Karachi (Hanafi)"));
   cmbITLMethod->addItem(QString("Islamic Society of North America"));
   cmbITLMethod->addItem(QString("Muslim World League (MWL)"));
   cmbITLMethod->addItem(QString("Umm Al-Qurra, Saudi Arabia"));
   cmbITLMethod->addItem(QString("Fixed Ishaa Interval (always 90)"));
}

void ConfigDialog::loadConfig(KPrayerTimeConfig & config)
{
    leLatitude->setText(QLocale().toString(config.m_latitude));
    leLongitude->setText(QLocale().toString(config.m_longitude));
    leElevation->setText(QLocale().toString(config.m_local_elevation));
    cmbTimeZone->setCurrentIndex(cmbTimeZone->findText(QString::number(config.m_time_zone)));
    cmbITLMethod->setCurrentIndex(config.m_itl_method - 1);
    sbErrorCorrMin->setValue(config.m_error_correction_min);
    sbErrorClockMin->setValue(config.m_error_clock_min);
    sbZawaalMin->setValue(config.m_zawaal);
    cbArabicNumerals->setChecked(config.m_arabic_numerals);
    cbInDST->setChecked(config.m_dst);
    ispNotifyMinutes->setValue(config.m_notify_minutes);
}

void ConfigDialog::saveConfig(KPrayerTimeConfig & config)
{
    config.m_latitude = QLocale().toDouble(leLatitude->text());
    config.m_longitude = QLocale().toDouble(leLongitude->text());
    config.m_local_elevation = QLocale().toDouble(leElevation->text());
    config.m_time_zone = cmbTimeZone->currentText().toFloat();
    config.m_itl_method = cmbITLMethod->currentIndex() + 1;
    config.m_error_correction_min = sbErrorCorrMin->value();
    config.m_error_clock_min = sbErrorClockMin->value();
    config.m_zawaal = sbZawaalMin->value();
    config.m_arabic_numerals = cbArabicNumerals->isChecked();
    config.m_dst = cbInDST->isChecked();
    config.m_notify_minutes = ispNotifyMinutes->value();
}
