/***************************************************************************
    copyright            : (C) 2009 by Abdulhaq Lynch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kprayertimeconfig.h"

#include <QtDebug>
#include <QSettings>
#include <QMessageBox>

KPrayerTimeConfig::KPrayerTimeConfig()
{
    m_settings = new QSettings("AbdulhaqSoftware", "KPrayertime");
    m_latitude = 0;
    m_longitude = 0;
    m_local_elevation = 0;
    m_itl_method = 3;
    m_error_correction_min = 2;
    m_error_clock_min = 2;
    m_zawaal = 5;
    m_dst = true;
    m_notify_minutes = 15;
}

KPrayerTimeConfig::~KPrayerTimeConfig()
{
    delete m_settings;
}

void KPrayerTimeConfig::loadConfig()
{
    m_latitude = m_settings->value("location/latitude").toDouble();
    m_longitude = m_settings->value("location/longitude").toDouble();
    m_local_elevation = m_settings->value("location/elevation").toInt();
    m_time_zone = m_settings->value("location/time_zone").toDouble();
    m_dst = m_settings->value("location/dst").toBool();
    m_itl_method = m_settings->value("method/itl_method").toInt();
    m_error_correction_min = m_settings->value("corrections/atmos").toInt();
    m_error_clock_min = m_settings->value("corrections/clock").toInt();
    m_zawaal = m_settings->value("corrections/zawaal").toInt();
    m_arabic_numerals = m_settings->value("language/arabic_numerals").toBool();
    m_notify_minutes = m_settings->value("notify/minutes").toInt();

    if (m_itl_method == 0) {
    	m_itl_method = 1;
    	m_notify_minutes = 15;
    }
}

void KPrayerTimeConfig::saveConfig()
{
	if (! m_settings->isWritable()) {
		QMessageBox::warning(0, QString("KPrayertime"),
				QString("Unable to write to configuration file at location:") + m_settings->fileName());
	    }
    m_settings->setValue("location/latitude", m_latitude);
    m_settings->setValue("location/longitude", m_longitude);
    m_settings->setValue("location/elevation", m_local_elevation);
    m_settings->setValue("location/time_zone", m_time_zone);
    m_settings->setValue("location/dst", m_dst);
    m_settings->setValue("method/itl_method", m_itl_method);
    m_settings->setValue("corrections/atmos", m_error_correction_min);
    m_settings->setValue("corrections/clock", m_error_clock_min);
    m_settings->setValue("corrections/zawaal", m_zawaal);
    m_settings->setValue("language/arabic_numerals", m_arabic_numerals);
    m_settings->setValue("notify/minutes", m_notify_minutes);
    m_settings->sync();
    qDebug() << "file name is" << m_settings->fileName();
    int settings_status = m_settings->status();
    if (settings_status == QSettings::AccessError) {
    	QMessageBox::warning(0, QString("KPrayertime"),
    					QString("Unable to write to configuration file: AccessError"));
    }
    if (settings_status == QSettings::FormatError) {
    	QMessageBox::warning(0, QString("KPrayertime"),
    					QString("Unable to write to configuration file: FormatError"));
    }
}
