/***************************************************************************
    copyright            : (C) 2009 by Abdulhaq Lynch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; Version 2 of the License.               *
 *                                                                         *
 ***************************************************************************/

#ifndef Painter_HEADER
#define Painter_HEADER

#include <QPainter>
#include <QTime>

#include "calctime.h"
#include "config.h"
#include "kprayertimeconfig.h"

enum SweepDirection {FORWARD, REVERSE};

enum DrawTimeMode {PLAIN, APPROACHING, NOW};

class SweepAngle {
public:
	float m_start_angle, m_end_angle;
	QColor m_start_colour, m_half_colour;
	SweepDirection m_direction;

	SweepAngle();
	SweepAngle(float start_angle, float end_angle,
			QColor start_colour, QColor m_half_colour,
			SweepDirection direction);
};


class Painter
{
    public:
    	Painter(int width, int height);
        ~Painter();

        void paint(QPainter *p, Prayer schedule[], QTime current_time, KPrayerTimeConfig & config);

        QRectF m_animate_coords;
    	float m_width;
    	float m_height;

    private:

        QString m_font_family;

    	float m_top_left_x;
    	float m_top_left_y;
    	float m_diameter;
    	float m_radius;

    	float m_origin_x;
    	float m_origin_y;

    	QPixmap * m_inner_pies;

    	void drawControls(QPainter *p);

        void getTangentialPoint(float radius, float angle, QPointF & point);

    	QString convertToArabic(QString numerals);

    	QString getStringTime(Prayer time, bool arabic_numerals);

    	void drawTimeArrow(QPainter *p, QTime current_time);

    	void drawSweep(QPainter *p, QTime current_time, SweepAngle sweep_angles[10]);

    	void drawTimeNumerals(QPainter *p, QTime current_time);

    	void drawInnerPies(float * pie_angles);

    	void showInnerPies(QPainter * p);

    	void drawArc(QPainter *p, float origin_x, float origin_y,
    			float from_angle, float to_angle);

    	void drawTimes(QPainter *p, Prayer * schedule, float * angles,
    			SweepAngle sweep_angles[12],
    			 bool arabic_numerals, QTime current_time);

    	DrawTimeMode getDrawTimeMode(int ix, SweepAngle sweep_angles[10],
    			QTime current_time);

    	bool timeInSweep(QTime time, SweepAngle sweep_angle);

    	void drawSalatTime(QPainter * p, float angle, float x,
    			QString text, DrawTimeMode draw_time_mode, bool offset_below_line);

    	void calcAngles(float * angles, Prayer * schedule);

    	void calcSweepAngles(float * angles, SweepAngle sweep_angles[10]);

    	void drawSweepShape(QPainter *p, float start_angle, float end_angle,
    			QColor colour, QColor half_colour);

    	void arcTo(QPainterPath & p, float from_angle, float to_angle, float radius);

    	void arcToWithCenter(QPainterPath & p, float x, float y,
    			float from_angle, float to_angle, float radius);

    	void drawSettings(QPainter *p, KPrayerTimeConfig & config);
};

class Arc {

public:

	Arc(QPointF point_a, QPointF point_b, float radius);

	float m_x, m_y;
	float m_radius;
	float m_start_angle, m_end_angle;

};

#endif
