/***************************************************************************
    copyright            : (C) 2009 by Abdulhaq Lynch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; Version 2 of the License.               *
 *                                                                         *
 ***************************************************************************/

#include "paintwidget.h"

#include <QTime>
#include <QTimer>
#include <QBrush>
#include <QtDebug>
#include <QMenu>
#include <QDialog>
#include <QVBoxLayout>
#include <QDialogButtonBox>

PaintWidget::PaintWidget()
{
    m_painter = 0;

    m_mode = NORMAL;
    m_last_minute = 1000;
    m_animate_hour = 0;
    m_animate_minute = 0;

    setWindowTitle("KPrayerTime");
    resize(400, 500);

    m_config = new KPrayerTimeConfig();
    m_config->loadConfig();
    m_config->m_font_family = "Arial";
    m_config_changed = true;

    QDate today = QDate::currentDate();
    makeSchedule(today);

    m_schedule = m_calc_time.m_prayer_times;

    connect(&timer, SIGNAL(timeout()), this, SLOT(timer_fired()));
    timer.start(1000);
}

void PaintWidget::makeSchedule(QDate date)
{
    m_calc_time.loadConfig(*m_config);
    m_calc_time.setDate(date);
    m_calc_time.makeSchedule();
}

void PaintWidget::timer_fired()
{
    if (m_mode == ANIMATE) update();
    else
    {
        QTime now = QTime::currentTime();
        int minute = now.minute();
    //    qDebug() << "timer_fired min:" << minute << "lastmin "<< m_last_minute ;
        if (minute != m_last_minute) update();
        m_last_minute = minute;
    }
}

void PaintWidget::paintEvent(QPaintEvent * event)
{
    paint();
}

void PaintWidget::mousePressEvent(QMouseEvent * event)
{
    if (m_painter->m_animate_coords.contains(event->pos()))
    {
        qDebug() << "hit";
        toggleAnimate();
    }
}

void PaintWidget::contextMenuEvent(QContextMenuEvent * event)
{
    qDebug() << "cm";
    QMenu context_menu(this);
    QAction * show_configuration = new QAction("Configure", this);
    connect(show_configuration, SIGNAL(triggered()),
            this, SLOT(show_configuration()));
    context_menu.addAction(show_configuration);
    context_menu.exec(event->globalPos(), show_configuration);
}

void PaintWidget::show_configuration()
{
    QDialog * dialog = new QDialog(this);
    QVBoxLayout * layout = new QVBoxLayout();
    dialog->setLayout(layout);
    m_config_dialog = new ConfigDialog();
    layout->addWidget(m_config_dialog);
    QDialogButtonBox * button_box = new QDialogButtonBox(
                            QDialogButtonBox::Ok
                          | QDialogButtonBox::Cancel);
    layout->addWidget(button_box);

    connect(button_box, SIGNAL(accepted()), dialog, SLOT(accept()));
    connect(button_box, SIGNAL(rejected()), dialog, SLOT(reject()));

    m_config_dialog->loadConfig(*m_config);
    int res = dialog->exec();
    if (res == QDialog::Accepted) {
        m_config_dialog->saveConfig(*m_config);
        m_config->saveConfig();
        m_config_changed = true;
    }
    delete m_config_dialog;
    delete layout;
    delete dialog;

    QDate today = QDate::currentDate();
    makeSchedule(today);
    update();
}

void PaintWidget::toggleAnimate()
{
    if (m_mode == NORMAL)
    {
        m_mode = ANIMATE;
        m_animate_hour = 0;
        m_animate_minute = 0;
    }
    else {
        m_mode = NORMAL;
        m_last_minute = 1000;
    }
    qDebug() << "mode set to " << m_mode;
}

void PaintWidget::paint()
{
    QPainter p(this);
    p.fillRect(0, 0, width(), height(), QBrush(Qt::black));
    p.setRenderHint(QPainter::Antialiasing);
    p.setRenderHint(QPainter::SmoothPixmapTransform);
    if (! m_painter || m_painter->m_width != width() || m_painter->m_height != height()
        || m_config_changed)
    {
        if (m_painter) delete m_painter;
        m_painter = new Painter(width(), height());
        m_config_changed = false;
    }
    QTime now = QTime::currentTime();
    if (m_mode == NORMAL) m_painter->paint(&p, m_schedule, now, *m_config);
    else
    {
        m_animate_minute += 20;
        if (m_animate_minute == 60) {
            m_animate_hour++;
            m_animate_minute = 0;
        }
        if (m_animate_hour == 24) m_animate_hour = 0;
        QTime animate_time(m_animate_hour, m_animate_minute);
        qDebug() << "paint animate for time " << animate_time;
        m_painter->paint(&p, m_schedule, animate_time, *m_config);
    }
}

