/***************************************************************************
    copyright            : (C) 2009 by Abdulhaq Lynch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; Version 2 of the License.               *
 *                                                                         *
 ***************************************************************************/

#include "plasma-kprayertime4.h"
#include "config.h"
#include "prayconst.h"

K_GLOBAL_STATIC_WITH_ARGS(KComponentData, kprayertimeComponent,
				    ("plasma-applet-kprayertime4",QByteArray(),
				    		KComponentData::SkipMainComponentRegistration))

#include <QPainter>
#include <QMessageBox>



Plasma_KPrayertime4::Plasma_KPrayertime4(
    QObject *parent, const QVariantList &args) : Plasma::Applet(parent, args)
{
    m_mode = NORMAL;
    setBackgroundHints(TranslucentBackground);
    resize(500, 500);
    setHasConfigurationInterface(true);
    m_painter = 0;

	m_salat_names[0] = QString("Fajr");
	m_salat_names[2] = QString("Dhuhr");
	m_salat_names[3] = QString("Asr");
	m_salat_names[4] = QString("Maghrib");
	m_salat_names[5] = QString("Ishaa'");

	m_arabic_salat_names[0] = QString::fromUtf8("الفجر");
	m_arabic_salat_names[2] = QString::fromUtf8("الظهر");
	m_arabic_salat_names[3] = QString::fromUtf8("العصر");
	m_arabic_salat_names[4] = QString::fromUtf8("المغرب");
	m_arabic_salat_names[5] = QString::fromUtf8("العشاء");
}

Plasma_KPrayertime4::~Plasma_KPrayertime4()
{
	delete icon;
	m_animate_timer->stop();
	delete m_animate_timer;
	if( m_painter) delete m_painter;
	delete m_config;
}

void Plasma_KPrayertime4::init()
{
	icon = new KIcon("kprayertime-icon");
	setBackgroundHints(TranslucentBackground);
	m_config = new KPrayerTimeConfig();
	loadConfig();
    m_config->m_font_family = "FreeSans";
    m_config_changed = true;
    m_calc.loadConfig(*m_config);
    QDate today = QDate::currentDate();
    makeSchedule(today);
	m_animate_timer = new QTimer(this);
	connect(m_animate_timer, SIGNAL(timeout()), this, SLOT(animateStep()));

    dataEngine("time")->connectSource("Local", this, 10000);
}

void Plasma_KPrayertime4::dataUpdated(const QString &source,
						const Plasma::DataEngine::Data &data)
{
    Q_UNUSED(source);

    if (m_mode == ANIMATE) return;

    kDebug() << "data updated";

    m_time = data["Time"].toTime();
    m_date = data["Date"].toDate();
    if (m_time.minute() != m_lastTimeSeen.minute()) {
        m_lastTimeSeen = m_time;
        refresh();
        checkNotifications(m_time);
    }
}

void Plasma_KPrayertime4::refresh()
{
	qDebug() << "refresh";
    m_calc.loadConfig(*m_config);
    QDate today = QDate::currentDate();
    makeSchedule(today);
    m_schedule = m_calc.m_prayer_times;
    update();
}

void Plasma_KPrayertime4::makeSchedule(QDate date)
{
	m_calc.setDate(date);
    m_calc.makeSchedule();
    m_schedule = m_calc.m_prayer_times;
}

void Plasma_KPrayertime4::loadConfig()
{
    m_config->loadConfig();

    if (m_config->m_latitude == 90 or m_config->m_longitude == 90) {
    	setConfigurationRequired(true,
    			"You must enter the correct latitude and longitude");
    }
    else setConfigurationRequired(false);
}

void Plasma_KPrayertime4::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED(option);

    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);
	p->setRenderHint(QPainter::TextAntialiasing);

    m_width = (int) contentsRect.width();
    m_height = (int) contentsRect.height();

    if (
        not m_painter || (m_painter->m_width != m_width)
        || (m_painter->m_height != m_height)
        || m_config_changed
    	) {
    	delete m_painter;
        m_painter = new Painter(m_width, m_height);
        m_config_changed = false;
    }

	QTime time;
	if (m_mode == NORMAL) time = m_time;
	else time = QTime(m_animate_hour, m_animate_minute);
    m_painter->paint(p, m_schedule, time, *m_config);
}

void Plasma_KPrayertime4::createConfigurationInterface(KConfigDialog* parent)
{
   dialog = new ConfigDialog();
   dialog->loadConfig(*m_config);
   parent->setButtons(KDialog::Ok | KDialog::Apply | KDialog::Cancel);
   KPageWidgetItem* page = parent->addPage(dialog, parent->windowTitle(),
													   QString("kprayertime"));
   page->setIcon(*icon);

   KNotifyConfigWidget* notify_config_widget = new KNotifyConfigWidget(parent);
   notify_config_widget->setApplication("plasma-applet-kprayertime4",
												   QString::null, QString::null);
   KPageWidgetItem *notifyPage = parent->addPage(notify_config_widget,
									   i18n("Notifications"), "Notifications");
   notifyPage->setIcon(KIcon("preferences-desktop-notification"));

   connect(parent, SIGNAL(applyClicked()), notify_config_widget, SLOT(save()));
   connect(parent, SIGNAL(okClicked()), notify_config_widget, SLOT(save()));

   connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
   connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
   connect(parent, SIGNAL(cancelClicked()), this, SLOT(configRejected()));
}

void Plasma_KPrayertime4::saveConfiguration() {
	dialog->saveConfig(*m_config);
    m_config->saveConfig();
    m_config_changed = true;
}

void Plasma_KPrayertime4::configAccepted()
{
	qDebug() << "config accepted";
	saveConfiguration();
	refresh();
	delete dialog;
}

void Plasma_KPrayertime4::configRejected()
{
	qDebug() << "config rejected";
	delete dialog;
}

void Plasma_KPrayertime4::mousePressEvent(QGraphicsSceneMouseEvent * event)
{
	QPointF pos = event->pos();
	kDebug() << "mouse press at " << pos;
	if (m_painter->m_animate_coords.contains(pos)) {
		kDebug() << " hit ";
		toggleAnimate();
	}
	else {
		kDebug() << "animate button at " << m_painter->m_animate_coords;
	}
}

void Plasma_KPrayertime4::animateStep()
{
	m_animate_minute += 20;
	if (m_animate_minute == 60) {
		m_animate_minute = 0;
		m_animate_hour += 1;
		if (m_animate_hour == 24) m_animate_hour = 0;
	}
	update();
}

void Plasma_KPrayertime4::toggleAnimate() {
	if (m_mode == NORMAL) {
		m_mode = ANIMATE;
		m_animate_hour = 0;
		m_animate_minute = 0;
		m_animate_timer->start(1000);
		displayApproachingNotification(3);
	}
	else {
		m_animate_timer->stop();
		m_mode = NORMAL;
		update();
	}
}

void Plasma_KPrayertime4::checkNotifications(QTime time) {
	for (int i=0; i<6; i++) {
		if (i==1) continue;
		if (time.hour() == m_schedule[i].hour &&
				time.minute() == m_schedule[i].minute) {
			displayNotification(i);
		}
		QTime time_approaching = time.addSecs(m_config->m_notify_minutes * 60);
		if (time_approaching.hour() == m_schedule[i].hour &&
				time_approaching.minute() == m_schedule[i].minute) {
					displayApproachingNotification(i);
				}
	}
}

void Plasma_KPrayertime4::displayNotification(int index) {
	qDebug() << "notification";
	QString msg = getNotificationMessage(index);
	QString arabic_msg = getArabicNotificationMessage(index);
	QString event_name("prayer_time");
	if (index == 0) event_name = QString("prayer_time_fajr");
	KNotification *notification= new KNotification(event_name, 0L,
				KNotification::Persistent);
	notification->setText(msg + QString("\n") + arabic_msg);
	notification->setPixmap(icon->pixmap(32,32));
	notification->setComponentData(*kprayertimeComponent);
	notification->sendEvent();
}

void Plasma_KPrayertime4::displayApproachingNotification(int index) {
//	QMessageBox::warning(0L, QString("App"), QString("App"));
	qDebug() << "approaching notification";
	QString event_name("prayer_time_approaching");
	QString msg = QString("The time for %1 salat is approaching.").arg(m_salat_names[index]);
	QString arabic_msg = QString::fromUtf8("اقترب وقت صلاة") + QString(" ") + m_arabic_salat_names[index];
	KNotification *notification= new KNotification(event_name, 0L,
											KNotification::Persistent);
	notification->setText(msg + QString("\n") + arabic_msg);
	notification->setPixmap(icon->pixmap(32,32));
	notification->setComponentData(*kprayertimeComponent);
	notification->sendEvent();
}

QString Plasma_KPrayertime4::getNotificationMessage(int index) {
	return QString("The time for %1 salat has arrived.").arg(m_salat_names[index]);
}

QString Plasma_KPrayertime4::getArabicNotificationMessage(int index) {
	return QString::fromUtf8("حان وقت صلاة") + QString(" ") + m_arabic_salat_names[index];
}

Qt::Orientations Plasma_KPrayertime4::expandingDirections() const
{
    return 0;
}

#include "plasma-kprayertime4.moc"
