/***************************************************************************
    copyright            : (C) 2009 by Abdulhaq Lynch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; Version 2 of the License.               *
 *                                                                         *
 ***************************************************************************/

#ifndef Plasma_KPrayertime4_HEADER
#define Plasma_KPrayertime4_HEADER

#include <QGraphicsSceneMouseEvent>
#include <QTimer>
#include <QSizeF>

#include <Plasma/Applet>
#include <Plasma/DataEngine>
#include <Plasma/Svg>

#include <KConfigDialog>
#include <KStandardDirs>
#include <KIcon>
#include <KNotification>
#include <KComponentData>
#include <KNotifyConfigWidget>

//#include <phonon>

#include "calctime.h"
#include "painter.h"

#include "ui_config.h"
#include "config.h"
#include "kprayertimeconfig.h"
#include "notification_widget.h"
 
enum Mode { NORMAL, ANIMATE };

 
class Plasma_KPrayertime4 : public Plasma::Applet
{
    Q_OBJECT
    public:
    	Plasma_KPrayertime4(QObject *parent, const QVariantList &args);
        ~Plasma_KPrayertime4();
        
        void paintInterface(QPainter *painter,
                const QStyleOptionGraphicsItem *option,
                const QRect& contentsRect);
        void init();
        QSizeF contentSizeHint() const;
        Qt::Orientations expandingDirections() const;
        void makeSchedule(QDate date);

    public slots:
        void refresh();
        void configAccepted();
        void configRejected();
        void dataUpdated(const QString &source, const Plasma::DataEngine::Data &data);
        void animateStep();

    protected:
        void createConfigurationInterface(KConfigDialog *parent);
        void mousePressEvent(QGraphicsSceneMouseEvent * event); 

    private:
        void loadConfig();
        void toggleAnimate();
//        void soundAdhaan(int index);
        void displayNotification(int index);
        void displayApproachingNotification(int index);
        QString getNotificationMessage(int index);
        QString getArabicNotificationMessage(int index);
        void checkNotifications(QTime time);
        
        KNotifyConfigWidget* notify_config_widget;

        KIcon *icon;

        int m_width, m_height;
        Mode m_mode;
        int m_animate_hour, m_animate_minute;
        QTimer * m_animate_timer;
        
        KPrayerTimeConfig * m_config;
        bool m_config_changed;
        
        Painter* m_painter;
        
        QTime m_time;
        QDate m_date;
        QTime m_lastTimeSeen;
        
    	// Object to calculate salat times
    	CalcTime m_calc;
    	// Pointer to salat times in array in calcTime
    	Prayer * m_schedule;

    	QString m_salat_names[7];
    	QString m_arabic_salat_names[7];
        
        ConfigDialog* dialog;

    private slots:
		void saveConfiguration();

};
 
// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_APPLET(plasma-applet-kprayertime4, Plasma_KPrayertime4)
#endif 
